# call pipeline and evaluate

import argparse
from utils import _load_config
from LLM_PostHocPipeline import Pipeline
from FaithfulnessPipeline import runFaithfulnessPipeline


explanation_list = ['lime', 'itg']  # ['lime', 'grad', 'sg', 'ig', 'shap', 'itg', 'random']
LLM_top_k = {'compas': 6, 'adult': 13, 'credit': 10}

for exp in explanation_list:

	# Load pipeline config/prompts
	config  = _load_config('LLM_pipeline_config.json')
	config['icl_params']['explanation_method'] = exp
	prompts = _load_config('prompts.json')

	# Run pipeline
	pipeline = Pipeline(config = config, prompts = prompts)
	folder_name_exp_id = pipeline.run()

	# Load faithfulness config
	faithfulness_config = _load_config('faithfulness_config.json')

	faithfulness_config['data_name']      = config['data_name']
	faithfulness_config['model_name']     = config['model_name']
	faithfulness_config['data_scaler']    = config['data_scaler']
	faithfulness_config['base_model_dir'] = config['base_model_dir']
	faithfulness_config['eval_min_idx']   = config['eval_min_idx']
	faithfulness_config['eval_max_idx']   = config['eval_max_idx']
	faithfulness_config['eval_max_k'] = LLM_top_k[config['data_name']]
	faithfulness_config['LLM_top_k'] = LLM_top_k[config['data_name']]
	faithfulness_config['output_dir']     = "./outputs/LLM_QueryAndReply/" + folder_name_exp_id + "/"
	faithfulness_config['LLM_topks_file_name'] = 'n_shot_' + str(config['n_shot']) + '_' + \
                                             config['data_name'] + '_' + config['model_name'] + '_LLM_topK.pkl'
	print(f'Experiment ID: {folder_name_exp_id}')
	print(f'Explanation method: {exp}')
	print(f"Model name: {config['model_name']}")
	runFaithfulnessPipeline(config = faithfulness_config)
